#ifndef __KdTreeApp_H__
#define __KdTreeApp_H__

#include <OgreRenderTargetListener.h>
#include <ExampleApplication.h>
#include <ivreader.h>
#include "TestKdTreeAppListener.h"


class KdTreeAppRenderTargetListener : public RenderTargetListener
{
public:
	KdTreeAppRenderTargetListener(SceneManager *sceneMgr);

protected:
	void preViewportUpdate (const RenderTargetViewportEvent &evt);
	void postRenderTargetUpdate (const RenderTargetEvent &evt);

	SceneManager *mSceneMgr;

	ShadowTechnique mSavedShadowTechnique;
	ColourValue mSavedAmbientLight;
};

class KdTreeApp : public ExampleApplication
{
public:

	KdTreeApp() :
	mFrameListener(0),
	mRenderTargerListener(0)
	{
	}

	KdTreeApp(const KdTreeAppListener::Options& options) :
	mFrameListener(0), 
	mRenderTargerListener(0),
	mOptions(options)
	{
	}

	~KdTreeApp()
	{
		delete mRenderTargerListener;
		delete mFrameListener;
	}

	// save entities to ascii file
	bool saveSceneASCII(const String& filename, SceneNode *entityroot);
	void createTerrainScene(Real max_x, Real max_y, Real max_z);
	Vector3 clampToGround(const Vector3& vect, Real offset, Real cap);

	const String& getViewCellsFileName() const { return mViewCells; }
protected:
	KdTreeAppListener *mFrameListener;
	KdTreeAppRenderTargetListener *mRenderTargerListener;
	SceneNode *mCamNode;
	SceneNode *mMoveNode;
	Camera *mTopCam;
	//Camera *mFollowCam;

	// scene options
	String  mSceneFiles;
	String	mViewCells;
	Real	mRotationRadius;
	int		mModelSpacing;
	int		mModelCount;
	int		mRandomCount;
	Real	mPlaneDim;
	bool	mShadowsEnabled;
	int		mSelectEntities;

	KdTreeAppListener::Options mOptions;

	virtual void setupResources(void);

	virtual void createScene(void);

	virtual void createFrameListener(void);
	
	virtual void chooseSceneManager(void);

    virtual void createCamera(void);

	virtual void createViewports(void);

    /** Configures the application - returns false if the user chooses to abandon configuration. */
    virtual bool configure(void);

	/** load scene from file **/
	bool loadScene(const String& filename);
	bool loadSceneIV(const String &filename, SceneNode *root, const int index);
	// loadding world geometry from cfg file - for terrain
	bool loadSceneCfg(const String &filename, SceneNode *root, const int index);
	// load entities from ascii file
	bool loadSceneASCII(const String &filename, SceneNode *root, const int index);

	typedef std::list<SceneNode *> NodeList;
	void addNodesToList(SceneNode* node, NodeList& list);

	void createMaterials(void);
	void createSimpleScene(void);

	RaySceneQuery* mRaySceneQuery;
private:
	std::string cat(std::string name, int x, int z)
	{
		std::stringstream s;
		s.fill('0');
		s << name;
		s.width(2);
		s << x;
		s.width(2);
		s << z;
		return s.str();
	}
};

#endif